// src/index.tsx
import { forwardRef } from "react";

// src/useCustomScroller.ts
import { useLayoutEffect, useRef, useState, useCallback } from "react";
var OS_SCROLLBAR_WIDTH = (() => {
  const outer = document.createElement("div");
  const inner = document.createElement("div");
  outer.style.overflow = "scroll";
  outer.style.width = "100%";
  inner.style.width = "100%";
  document.body.appendChild(outer);
  outer.appendChild(inner);
  const scrollbarWidth = outer.offsetWidth - inner.offsetWidth;
  outer.removeChild(inner);
  document.body.removeChild(outer);
  return scrollbarWidth;
})();
var SCROLLBAR_WIDTH = OS_SCROLLBAR_WIDTH || 20;
function useCustomScroller(customRef, { disabled } = { disabled: false }) {
  const [scrollRatio, setScrollRatio] = useState(1);
  const [isDraggingTrack, setIsDraggingTrack] = useState(false);
  const ref = useRef(null);
  const scrollerRef = customRef || ref;
  const trackRef = useRef(null);
  const trackAnimationRef = useRef(0);
  const memoizedProps = useRef();
  useLayoutEffect(() => {
    const el = scrollerRef.current;
    if (!el) return;
    let scrollbarAnimation;
    const updateScrollbar = () => {
      cancelAnimationFrame(scrollbarAnimation);
      scrollbarAnimation = requestAnimationFrame(() => {
        var _a, _b;
        const { clientHeight, scrollHeight } = el;
        setScrollRatio(clientHeight / scrollHeight);
        memoizedProps.current = {
          clientHeight,
          scrollHeight,
          trackHeight: (_b = (_a = trackRef.current) == null ? void 0 : _a.clientHeight) != null ? _b : 0
        };
      });
    };
    const resizeObserver = new ResizeObserver(updateScrollbar);
    const observeScroller = () => {
      resizeObserver.observe(el);
      for (const node of el.children) {
        resizeObserver.observe(node);
      }
    };
    observeScroller();
    const mutationObserver = new MutationObserver(() => {
      resizeObserver.disconnect();
      observeScroller();
      updateScrollbar();
    });
    mutationObserver.observe(el, { childList: true, subtree: true });
    window.addEventListener("resize", updateScrollbar);
    updateScrollbar();
    return () => {
      cancelAnimationFrame(scrollbarAnimation);
      window.removeEventListener("resize", updateScrollbar);
      resizeObserver.disconnect();
      mutationObserver.disconnect();
    };
  }, [scrollerRef]);
  useLayoutEffect(() => {
    if (!disabled) return;
    const el = scrollerRef.current;
    if (!el) return;
    const onWheel = (e) => e.preventDefault();
    el.addEventListener("wheel", onWheel, { passive: false });
    return () => {
      el.removeEventListener("wheel", onWheel);
    };
  }, [scrollerRef, disabled]);
  const onScroll = useCallback(() => {
    if (scrollRatio === 1) return;
    const el = scrollerRef.current;
    const track = trackRef.current;
    if (!el || !track) return;
    cancelAnimationFrame(trackAnimationRef.current);
    trackAnimationRef.current = requestAnimationFrame(() => {
      if (!memoizedProps.current) return;
      const { clientHeight, scrollHeight, trackHeight } = memoizedProps.current;
      const ratio = el.scrollTop / (scrollHeight - clientHeight);
      const y = ratio * (clientHeight - trackHeight);
      track.style.transform = `translateY(${y}px)`;
    });
  }, [scrollerRef, scrollRatio]);
  const moveTrack = useCallback(
    (e) => {
      const el = scrollerRef.current;
      if (!el) return;
      let moveAnimation;
      let lastPageY = e.pageY;
      let lastScrollTop = el.scrollTop;
      setIsDraggingTrack(true);
      const drag = ({ pageY }) => {
        cancelAnimationFrame(moveAnimation);
        moveAnimation = requestAnimationFrame(() => {
          const delta = pageY - lastPageY;
          lastScrollTop += delta / scrollRatio;
          lastPageY = pageY;
          el.scrollTop = lastScrollTop;
        });
      };
      const stop = () => {
        setIsDraggingTrack(false);
        window.removeEventListener("mousemove", drag);
      };
      window.addEventListener("mousemove", drag);
      window.addEventListener("mouseup", stop, { once: true });
    },
    [scrollerRef, scrollRatio]
  );
  const wrapperProps = {
    style: {
      marginLeft: `-${SCROLLBAR_WIDTH}px`
    }
  };
  const scrollerProps = {
    ref: scrollerRef,
    onScroll: disabled ? void 0 : onScroll,
    style: {
      right: `-${SCROLLBAR_WIDTH}px`,
      padding: `0 ${SCROLLBAR_WIDTH}px 0 0`,
      width: `calc(100% + ${OS_SCROLLBAR_WIDTH}px)`
    }
  };
  const trackProps = {
    ref: trackRef,
    onMouseDown: disabled ? void 0 : moveTrack,
    style: {
      right: isDraggingTrack ? 1 : void 0,
      width: isDraggingTrack ? 10 : void 0,
      height: `${scrollRatio * 100}%`,
      opacity: isDraggingTrack ? 1 : void 0,
      display: disabled || scrollRatio === 1 ? "none" : void 0
    }
  };
  return [wrapperProps, scrollerProps, trackProps];
}

// src/index.module.css
var src_default = {
  main: "src_main",
  track: "src_track",
  wrapper: "src_wrapper",
  inner: "src_inner"
};

// src/index.tsx
import { jsx, jsxs } from "react/jsx-runtime";
var cx = (...args) => args.filter(Boolean).join(" ");
var CustomScroller = forwardRef(
  ({ scrollDisabled = false, className, innerClassName, children, ...props }, ref) => {
    const [wrapperProps, scrollerProps, trackProps] = useCustomScroller(
      ref !== null && "current" in ref ? ref : void 0,
      { disabled: scrollDisabled }
    );
    return /* @__PURE__ */ jsxs("div", { className: cx(className, src_default.main), ...props, children: [
      /* @__PURE__ */ jsx("div", { className: src_default.wrapper, ...wrapperProps, children: /* @__PURE__ */ jsx("div", { className: cx(innerClassName, src_default.inner), ...scrollerProps, children }) }),
      /* @__PURE__ */ jsx("div", { className: src_default.track, ...trackProps })
    ] });
  }
);
CustomScroller.displayName = "CustomScroller";
var src_default2 = CustomScroller;
export {
  src_default2 as default
};
//# sourceMappingURL=index.mjs.map